# Script PowerShell: setup-conf.ps1
# 1. Recupera i nomi delle cartelle
# 2. Riorganizza secondo le regole
# 3. Legge e aggiorna il JSON con una UI interattiva

Add-Type -AssemblyName System.Windows.Forms
Add-Type -AssemblyName System.Drawing

$ScriptDir = Split-Path -Parent $MyInvocation.MyCommand.Definition

$basePath = Resolve-Path (Join-Path $ScriptDir '../../../../pdca0')
$configPath = Resolve-Path (Join-Path $ScriptDir '../../../resources/pdca0-config.json')

# 1. Recupera i nomi delle cartelle
$dirs = Get-ChildItem -Path $basePath -Directory | Select-Object -ExpandProperty Name

# 2. Riorganizza gli input
$backend_modules = $dirs | Where-Object { $_ -like 'pdca0*' -and $_ -like '*v1' -and $_ -notlike '*deploy*' }
$core_module = $dirs | Where-Object { $_ -like 'core*' }
$xdce_modules = $dirs | Where-Object { $_ -like 'xdce-module*' -or $_ -like '*-ui' }
$deploy_module = $dirs | Where-Object { $_ -like '*deploy*' }

# 3. Leggi il file JSON
$json = Get-Content $configPath -Raw | ConvertFrom-Json

# Funzione per creare la UI dinamica
function Show-ConfigForm {
    param (
        [Parameter(Mandatory)] $json,
        [Parameter(Mandatory)] $modulesDict
    )

    $form = New-Object Windows.Forms.Form
    $form.Text = "Setup Darwin PDCA0 environments"
    $form.Size = New-Object Drawing.Size(600, 500)
    $form.StartPosition = "CenterScreen"

    $mainPanel = New-Object System.Windows.Forms.TableLayoutPanel
    $mainPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
    $mainPanel.RowCount = 2
    $mainPanel.ColumnCount = 1
    $mainPanel.RowStyles.Add((New-Object System.Windows.Forms.RowStyle([System.Windows.Forms.SizeType]::Percent, 100)))
    $mainPanel.RowStyles.Add((New-Object System.Windows.Forms.RowStyle([System.Windows.Forms.SizeType]::Absolute, 35)))
    $form.Controls.Add($mainPanel)

    $controls = @{}
    $y = 10

    $dictPanel = New-Object System.Windows.Forms.Panel
    $dictPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
    $dictPanel.BorderStyle = 'Fixed3D'
    $dictPanel.BackColor = 'White'
    $dictPanel.AutoScroll = $true
    $dictPanel.Anchor = [System.Windows.Forms.AnchorStyles]::Top -bor [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Left -bor [System.Windows.Forms.AnchorStyles]::Right
    $mainPanel.Controls.Add($dictPanel, 0, 0)

    foreach ($key in $modulesDict.Keys) {
        $label = New-Object Windows.Forms.Label
        $label.Text = $key
        $label.Location = New-Object Drawing.Point(10, $y)
        $label.Size = New-Object Drawing.Size(200, 20)
        $dictPanel.Controls.Add($label)
        $y += 20

        $values = $modulesDict[$key]
        $current = $json.PSObject.Properties[$key].Value
        if ($null -ne $current -and $current.GetType().IsArray) {
            # Array: Checkboxes
            $chkboxes = @()
            foreach ($val in $values) {
                $chk = New-Object Windows.Forms.CheckBox
                $chk.Text = $val
                $chk.Location = New-Object Drawing.Point(30, $y)
                $chk.Size = New-Object Drawing.Size(200, 20)
                if ($current -contains $val) { $chk.Checked = $true }
                $dictPanel.Controls.Add($chk)
                $chkboxes += $chk
                $y += 22
            }
            $controls[$key] = $chkboxes
        } elseif ($null -ne $current -and $current -is [string]) {
            # String: Radio in Panel
            $panel = New-Object Windows.Forms.Panel
            $panel.Location = New-Object Drawing.Point(25, $y)
            $panel.Size = New-Object Drawing.Size(250, ($values.Count * 24 + 8))
            $panel.BorderStyle = 'None'
            $radios = @()
            $panelY = 4
            foreach ($val in $values) {
                $rad = New-Object Windows.Forms.RadioButton
                $rad.Text = $val
                $rad.Location = New-Object Drawing.Point(4, $panelY)
                $rad.Size = New-Object Drawing.Size(200, 20)
                if ($current -eq $val) { $rad.Checked = $true }
                $panel.Controls.Add($rad)
                $radios += $rad
                $panelY += 22
            }
            $dictPanel.Controls.Add($panel)
            $controls[$key] = $radios
            $y += $panel.Height
        } else {
            # Default fallback: radio in Panel
            $panel = New-Object Windows.Forms.Panel
            $panel.Location = New-Object Drawing.Point(25, $y)
            $panel.Size = New-Object Drawing.Size(250, ($values.Count * 24 + 8))
            $panel.BorderStyle = 'None'
            $radios = @()
            $panelY = 4
            foreach ($val in $values) {
                $rad = New-Object Windows.Forms.RadioButton
                $rad.Text = $val
                $rad.Location = New-Object Drawing.Point(4, $panelY)
                $rad.Size = New-Object Drawing.Size(200, 20)
                $panel.Controls.Add($rad)
                $radios += $rad
                $panelY += 22
            }
            $dictPanel.Controls.Add($panel)
            $controls[$key] = $radios
            $y += $panel.Height
        }
        $y += 10
    }

    $buttonPanel = New-Object System.Windows.Forms.Panel
    $buttonPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
    $buttonPanel.Anchor = [System.Windows.Forms.AnchorStyles]::Top -bor [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Left -bor [System.Windows.Forms.AnchorStyles]::Right
    $buttonPanel.Height = 35
    $mainPanel.Controls.Add($buttonPanel, 0, 1)

    $btnSave = New-Object Windows.Forms.Button
    $btnSave.Text = "Salva"
    $btnSave.Location = New-Object Drawing.Point(490, 0)
    $btnSave.Size = New-Object Drawing.Size(80, 25)
    $btnSave.Anchor = [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Right
    $btnSave.Add_Click({
        foreach ($key in $modulesDict.Keys) {
            $controlsForKey = $controls[$key]
            if ($controlsForKey[0] -is [Windows.Forms.CheckBox]) {
                $selected = @($controlsForKey | Where-Object { $_.Checked } | ForEach-Object { $_.Text })
                $json | Add-Member -MemberType NoteProperty -Name $key -Value $selected -Force
            } else {
                $selected = $controlsForKey | Where-Object { $_.Checked } | Select-Object -First 1
                if ($selected) {
                    $json | Add-Member -MemberType NoteProperty -Name $key -Value $selected.Text -Force
                }
            }
        }
        $json | ConvertTo-Json -Depth 10 | Set-Content $configPath
        [System.Windows.Forms.MessageBox]::Show("Configurazione salvata!", "Salvato", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Information)
        $form.Close()
    })
    $buttonPanel.Controls.Add($btnSave)

    $form.ShowDialog()
}

$modulesDict = @{
    backend_modules = $backend_modules
    core_module = $core_module
    xdce_modules = $xdce_modules
    deploy_module = $deploy_module
}

Show-ConfigForm -json $json -modulesDict $modulesDict
